module clean_arrays
  use clean_def
  use image_def
  !
  ! Static variables first, allocatable next.
  !
  type (uvmap_par), save :: themap
  type (clean_par), target :: method, user_method
  type (gildas), save, target :: hbeam, hdirty, hclean, hresid, hweight
  type (gildas), save, target :: hprim, huvi, huv, huvt, hfields
  type (gildas), save, target :: hcct, hmask, hsky
  type (gildas), save :: huvm, hraw, hself, hagain, hbgain
  type (gildas), save :: hsingle     ! Single Dish Cube or Table
  !
  ! These are UV data
  real, pointer :: duv(:,:)
  logical, save :: uv_plotted
  ! For the Multi-Primary beams Mosaic deconvolution
  ! C_ stands for "Compact Array"
  type (clean_par), target :: c_method, cuse_method
  type (gildas), save :: c_hbeam, c_hdirty, c_hresid, c_hclean
  type (gildas), save :: c_hprim
  !
  real, allocatable, save :: g_weight(:)  ! UV weights
  real, allocatable, save :: g_v(:)       ! V values
  logical, save :: do_weig                ! Optimization of weight computing
  !
  ! Imaging and deconvolution arrays
  real, allocatable, target, save :: dbeam(:,:,:,:) ! 4-D
  real, allocatable, target, save :: ddirty(:,:,:)  ! Dirty 3-D 
  real, allocatable, target, save :: dclean(:,:,:)  ! Clean 3-D
  real, allocatable, target, save :: dresid(:,:,:)  ! Residual 3-D
  real, allocatable, target, save :: dprim(:,:,:,:) ! Primary beam 4-D
  real, allocatable, target, save :: dweight(:,:,:)   ! Mosaic weights 3-D
  real, allocatable, target, save :: dsky(:,:,:)      ! Primary beam corrected 3-D
  real, allocatable, target, save :: dfields(:,:,:,:) ! Mosaic fields 4-D
  !
  ! Deconvolution arrays
  real, allocatable, target, save :: dmask(:,:,:)   ! Input 3-D 0, #0 mask
  integer, allocatable, target, save :: d_list(:)   ! List of selected pixels
  logical, allocatable, target, save :: d_mask(:,:) ! Logical mask for pixel selection
  real, allocatable, target, save :: dcct(:,:,:)    ! Clean component values
  type (cct_par), allocatable, save :: tcc(:)       ! Clean component list
  !
  ! These are UV data
  real, allocatable, target, save :: duvi(:,:)  ! Original UV data
  real, pointer :: duvr(:,:)                    ! Resampled UV data
  real, pointer :: duvs(:,:)                    ! Sorted UV data
  real, allocatable, target, save :: duvt(:,:)  ! Time-Baseline sorted UV data, for UV_FLAG
  real, allocatable, target, save :: duvm(:,:)  ! Model UV data
  real, allocatable, target, save :: duvbg(:,:) ! Baseline Gains (UV data like)
  real, allocatable, target, save :: duvraw(:,:)   ! Raw UV data for Self calibration
  real, allocatable, target, save :: duvself(:,:)  ! Self calibrated UV data
  real, allocatable, target, save :: dagain(:,:)   ! Antenna based gain
  !
  ! Short spacing data array
  real, allocatable, target, save :: dsingle(:,:,:)
  !
  ! For the Multi-Primary beams Mosaic deconvolution
  ! C_ stands for "Compact Array"
  real, allocatable, target, save :: c_dbeam(:,:,:)
  real, allocatable, target, save :: c_ddirty(:,:,:)
  real, allocatable, target, save :: c_dclean(:,:,:)
  real, allocatable, target, save :: c_dresid(:,:,:)
  real, allocatable, target, save :: c_dprim(:,:,:)
  logical, allocatable, target, save :: c_mask(:,:)
  integer, allocatable, target, save :: c_list(:)
  real, allocatable, target, save :: c_weight(:,:,:)  ! 3-D
  !
  integer :: niter_listsize=0
  integer, allocatable :: niter_list(:)
  integer :: ares_listsize=0
  real, allocatable :: ares_list(:)
  ! List of selected fields in UV_MAP (used for UV_RESTORE and
  ! also for SHOW FIELDS)
  integer :: selected_fieldsize=0
  integer, allocatable :: selected_fields(:)
end module clean_arrays
!
module moment_arrays
  use image_def
  type(gildas), save :: hpeak, hmean, hvelo, hwidth
  real, allocatable, target, save :: dmean(:,:), dpeak(:,:), dvelo(:,:), dwidth(:,:)
end module moment_arrays
!
module clean_support
  use gkernel_types
  ! CLEAN\SUPPORT polygon, mapped on SUPPORT%
  type(polygon_t), save :: supportpol
  integer, save :: support_type  
  integer, parameter :: support_mask = 1 
  integer, parameter :: support_poly = -1
  integer, parameter :: support_none = 0
  character(len=varname_length), parameter :: supportvar='SUPPORT'
end module clean_support
!
module clean_types
  use gkernel_types
  integer mtype
  parameter(mtype=20)
  character(len=12) :: vtype(1:mtype) = (/ &
  &  'COVERAGE    ',  &
  &  'UV          ',  &
  &  'BEAM        ',  &
  &  'DIRTY       ',  &
  &  'CLEAN       ',  &
  &  'PRIMARY     ',  &
  &  'RESIDUAL    ',  &
  &  'MASK        ',  &
  &  'SUPPORT     ',  &
  &  'CCT         ',  &
  &  'MODEL       ',  &
  &  'SKY         ',  &
  &  'FIELDS      ',  &
  &  'CGAINS      ',  &
  &  'SELFCAL     ',  &
  &  'SINGLEDISH  ',  &
  &  'NOISE       ',  &
  &  'SNR         ',  &     ! NOISE and SNR are special, just for SHOW NOISE  
  &  'MOMENTS     ',  &     ! special, just for SHOW MOMENTS  
  &  'FIT         ' /)
  !
  character(len=12) :: etype(1:mtype) = (/ &
  & '.uvt        ',  &
  & '.uvt        ',  &
  & '.beam       ',  &
  & '.lmv        ',  &
  & '.lmv-clean  ',  &
  & '.lobe       ',  &
  & '.lmv-res    ',  &
  & '.msk        ',  &
  & '.pol        ',  &
  & '.cct        ',  &
  & '.uvt        ',  &
  & '.lmv-sky    ',  &
  & '.fields     ',  &
  & '.uvt        ',  &      
  & '.tab        ',  &
  & '.tab        ',  &
  & '            ',  &
  & '            ',  &
  & '            ',  &
  & '.uvfit      '  /)       
  !
  character(len=12) ctype
  !
  integer, parameter :: code_save_uv=2
  integer, parameter :: code_save_beam=3
  integer, parameter :: code_save_dirty=4
  integer, parameter :: code_save_clean=5
  integer, parameter :: code_save_primary=6
  integer, parameter :: code_save_resid=7
  integer, parameter :: code_save_mask=8
  integer, parameter :: code_save_support=9
  integer, parameter :: code_save_cct=10
  integer, parameter :: code_save_model=11
  integer, parameter :: code_save_sky=12
  integer, parameter :: code_save_fields=13
  integer, parameter :: code_save_gains=14
  integer, parameter :: code_save_single=15
  logical :: save_data(mtype)
  !
  ! Read / Write optimization
  type readop_t
    type(mfile_t) :: modif
    integer :: lastnc(2) = 0
    integer :: change = 0
  end type
  !
  type (readop_t), save :: optimize(mtype)
  !
  integer :: rw_optimize
end module clean_types
!
module clean_default
  use clean_def
  !
  real d_min,d_max                   ! min max of dirty image
  real major_sec
  real minor_sec
  !
  type (uvmap_par), save :: default_map, old_map, save_map
  !
  ! OPEN-MP control parameters
  logical, save :: omp_compiled = .false.
  logical, save :: omp_uvmap = .false.
  integer, save :: omp_grid = 0
  logical, save :: omp_restore_fast = .true.
  logical, save :: omp_uvmap_lock = .false.
  integer, save :: omp_single_beam = 0
  logical, save :: omp_debug = .false.
  !
  ! Parameter to select the imaging tool version. Non zero means old
  ! non-optimized code.
  integer, save :: map_version = 0
  !
  ! Parameters for determining image size
  integer, save :: map_power = 2         ! Max exponent of powers of 3 and 5
  real, save :: map_rounding = 0.05      ! Tolerance for rounding to floor
  !
  logical, save :: MappingError = .false. ! Error status from scripts
  !
  character(len=16), save :: last_shown  ! CLEAN or DIRTY
  character(len=16), save :: last_resid  ! CCT or UV_FIT
end module clean_default

module code_names
  use image_def
  character(len=12), save :: items(code_uvt_last)
  character(len=4), save :: stoke(-22:22)
end module code_names

module uvfit_data
  use image_def
  integer, parameter :: mpar=70      ! Max number of parameters
  integer, parameter :: mf=10        ! Max number of functions
  integer, parameter :: mpin=7       ! Max number of parameters per function
  real(kind=8) :: pars(mpar)         ! Parameter values
  real(kind=8) :: sw
  real, allocatable :: uvriw(:,:)    ! Work space for visibilities
  integer nf                         ! number of functions
  integer ifunc(mf)                  ! Function codes
  integer npfunc(mf)                 ! number of parameters per function
  logical :: savef(mf)               ! Subtract function
  integer npuvfit                    ! number of points
  integer npar                       ! Number of parameters
  integer nstart(mpar)               ! Number of starts per parameter
  !
  type (gildas) :: huvfit            ! Table of fitted parameters
  real, allocatable :: duvfit(:,:)
end module uvfit_data
!

